import logging
import os

DEBUG_MODE = os.getenv("DEBUG_MODE", False)

logging.basicConfig(level=logging.DEBUG if DEBUG_MODE else logging.INFO,
                    format='%(asctime)s - %(name)s - %(levelname)s - %(message)s')

logger = logging.getLogger(__name__)